@@ -62,6 +62,7 @@
 #define DOMAIN_SUFFIX_INDIEZEN ".indiezen.org"
 #define DOMAIN_SUFFIX_JEUX ".jeux.fr"
 #define DOMAIN_SUFFIX_QUAKENET ".quakenet.org"
+#define DOMAIN_SUFFIX_SPIDERNET ".spidernet.org"
 #define DOMAIN_SUFFIX_THUNDERCITY ".thundercity.org"
 #define DOMAIN_SUFFIX_UNDERNET ".undernet.org"
 
@@ -108,7 +109,7 @@
 #define NICK_AUTHSERV "AuthServ"
 
 #define NICK_CHANSERV "ChanServ"
-#define NICK_FREENODE_CONNECT "freenode-connect"
+#define NICK_FREENODE_CONNECT "frigg"
 #define NICK_FUNCOM_Q_SERVICE NICK_QUAKENET_Q "@cserve.funcom.com"
 #define NICK_GAMESURGE_AUTHSERV "AuthServ"
 #define NICK_GAMESURGE_AUTHSERV_SERVICE NICK_GAMESURGE_AUTHSERV "@Services.GameSurge.net"
@@ -147,6 +148,7 @@ typedef enum {
 	IRC_NETWORK_TYPE_DALNET    = 0x0800,
 	IRC_NETWORK_TYPE_FUNCOM    = 0x1000,
 	IRC_NETWORK_TYPE_INDIEZEN  = 0x2000,
+	IRC_NETWORK_TYPE_SPIDERNET = 0x4000
 } IRCHelperStateFlags;
 
 struct proto_stuff
@@ -249,6 +251,8 @@ static IRCHelperStateFlags get_connection_type(PurpleConnection *connection)
 		type = IRC_NETWORK_TYPE_UNDERNET;
 	else if (g_str_has_suffix(username, DOMAIN_SUFFIX_INDIEZEN))
 		type = IRC_NETWORK_TYPE_INDIEZEN;
+	else if (g_str_has_suffix(username, DOMAIN_SUFFIX_SPIDERNET))
+		type = IRC_NETWORK_TYPE_SPIDERNET;
 
 	g_free(username);
 	return type;
@@ -639,7 +643,7 @@ static void signed_on_cb(PurpleConnection *connection)
 
 			if (state & IRC_NETWORK_TYPE_THUNDERCITY)
 				nickserv_msg_identify("AUTH", connection->proto_data, connection, nickpassword);
-			else if (state & IRC_NETWORK_TYPE_INDIEZEN)
+			else if (state & (IRC_NETWORK_TYPE_INDIEZEN | IRC_NETWORK_TYPE_SPIDERNET))
 				nickserv_msg_identify("identify", connection->proto_data, connection, nickpassword);
 			else
 				nickserv_identify(connection->proto_data, connection, nickpassword);
@@ -785,7 +789,8 @@ get_auto_response(PurpleConnection *gc, const char *name)
 	return ar;
 }
 
-static gboolean receiving_im_msg_cb(PurpleAccount *account, gchar **sender, gchar **buffer,
+static gboolean receiving_im_msg_cb(PurpleAccount *account, gchar **sender,
+                                    gchar **buffer,
                                     PurpleConversation *conv,
                                     gint *flags, gpointer data)
 {
@@ -836,7 +841,8 @@ static gboolean receiving_im_msg_cb(PurpleAccount *account, gchar **sender, gcha
 	/* SIMPLE SUPPRESSION RULES */
 
 	/* Suppress the FreeNode stats collection bot. */
-	if (g_str_equal(nick, NICK_FREENODE_CONNECT))
+	if (g_str_equal(nick, NICK_FREENODE_CONNECT) &&
+	    g_str_has_prefix(msg, "Received CTCP &apos;VERSION&apos;"))
 	{
 		return TRUE;
 	}
