@@ -1,6 +1,6 @@
 /*
  * Switchspell - Switch spelling language during run time.
- * Copyright (C) 2007
+ * Copyright (C) 2007-2008
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
  * 02111-1301, USA.
  */
 
+/* If you can't figure out what this line is for, DON'T TOUCH IT. */
 #include "../common/pp_internal.h"
 
 #define PLUGIN_ID           "gtk-plugin_pack-switchspell"
@@ -34,7 +35,11 @@
 #include <gtkconv.h>
 #include <gtkplugin.h>
 
+#ifdef HAVE_ENCHANT
+#include <enchant.h>
+#else
 #include <aspell.h>
+#endif
 #include <gtkspell/gtkspell.h>
 #include <string.h>
 
@@ -80,8 +85,12 @@ menu_conv_use_dict_cb(GObject *m, gpointer data)
 
 	gtkconv = PIDGIN_CONVERSATION(conv);
 	spell = gtkspell_get_from_text_view(GTK_TEXT_VIEW(gtkconv->entry));
-	if (spell != NULL)
-		gtkspell_set_language(spell, lang, &error);  /* XXX: error can possibly leak here */
+	if (spell != NULL) {
+		if (!gtkspell_set_language(spell, lang, &error)) {
+			purple_debug_error("switchspell", "failed to set language %s: %s\n", lang, error->message);
+			g_error_free(error);
+		}
+	}
 	g_object_set_data(G_OBJECT(gtkconv->entry), PROP_LANG, lang);
 
 	node = blist_node_for_conv(gtkconv->active_conv);
@@ -89,17 +98,49 @@ menu_conv_use_dict_cb(GObject *m, gpointer data)
 		purple_blist_node_set_string(node, "switchspell", lang);
 }
 
+#ifdef HAVE_ENCHANT
+struct nufan
+{
+	GSList *group;
+	GtkWidget *menu;
+	PidginWindow *win;
+};
+
+static void
+enchant_dict_desc_cb(const char * const lang_tag, const char * const provider_name,
+                     const char * const provider_desc, const char * const provider_file,
+                     void *user_data)
+{
+	GtkWidget *menu = ((struct nufan *)user_data)->menu;
+	PidginWindow *win = ((struct nufan *)user_data)->win;
+
+	GtkWidget *menuitem = gtk_radio_menu_item_new_with_label(((struct nufan *)user_data)->group, lang_tag);
+	((struct nufan *)user_data)->group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
+	g_object_set_data(G_OBJECT(menuitem), "user_data", win);
+	g_object_set_data_full(G_OBJECT(menuitem), "lang", g_strdup(lang_tag), g_free);
+	g_signal_connect(G_OBJECT(menuitem), "activate",
+					G_CALLBACK(menu_conv_use_dict_cb), NULL);
+	gtk_widget_show(menuitem);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+}
+#endif
+
 static void
 regenerate_switchspell_menu(PidginConversation *gtkconv)
 {
 	PidginWindow *win;
 	GtkWidget *menu;
 	GtkWidget *mitem;
+#ifdef HAVE_ENCHANT
+	struct nufan user_data;
+	EnchantBroker *eb;
+#else
 	GSList *group = NULL;
-	AspellConfig * config;
-	AspellDictInfoList * dlist;
-	AspellDictInfoEnumeration * dels;
-	const AspellDictInfo * entry;
+	AspellConfig *config;
+	AspellDictInfoList *dlist;
+	AspellDictInfoEnumeration *dels;
+	const AspellDictInfo *entry;
+#endif
 
 	if (gtkconv == NULL)
 		return;
@@ -121,23 +162,32 @@ regenerate_switchspell_menu(PidginConversation *gtkconv)
 	menu = gtk_menu_new();
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(mitem), menu);
 
+#ifdef HAVE_ENCHANT
+	user_data.group = NULL;
+	user_data.menu = menu;
+	user_data.win = win;
+	eb = enchant_broker_init();
+	enchant_broker_list_dicts(eb, enchant_dict_desc_cb, &user_data);
+	enchant_broker_free(eb);
+#else
 	config = new_aspell_config();
 	dlist = get_aspell_dict_info_list(config);
 	delete_aspell_config(config);
 
 	dels = aspell_dict_info_list_elements(dlist);
+	aspell_dict_info_list_empty(dlist);
 	while ((entry = aspell_dict_info_enumeration_next(dels)) != 0) {
 		GtkWidget *menuitem = gtk_radio_menu_item_new_with_label(group, entry->name);
 		group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
 		g_object_set_data(G_OBJECT(menuitem), "user_data", win);
-		g_object_set_data(G_OBJECT(menuitem), "lang", (gchar *)entry->name);
+		g_object_set_data_full(G_OBJECT(menuitem), "lang", g_strdup((gchar *)entry->name), g_free);
 		g_signal_connect(G_OBJECT(menuitem), "activate",
 					G_CALLBACK(menu_conv_use_dict_cb), NULL);
 		gtk_widget_show(menuitem);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	}
-
 	delete_aspell_dict_info_enumeration(dels);
+#endif
 	gtk_widget_show_all(menu);
 }
 
@@ -146,7 +196,7 @@ update_switchspell_selection(PidginConversation *gtkconv)
 {
 	PidginWindow *win;
 	GtkWidget *menu;
-	GList *item;
+	GList *item, *items;
 	char *curlang;
 
 	if (gtkconv == NULL)
@@ -156,6 +206,10 @@ update_switchspell_selection(PidginConversation *gtkconv)
 	if (win == NULL)
 		return;
 
+	/* Leave the menu alone if the tab is not the active conversation */
+	if (pidgin_conv_window_get_active_gtkconv(win) != gtkconv)
+		return;
+
 	menu = g_object_get_data(G_OBJECT(win->window), PROP_LANG);
 	if (menu == NULL)
 		return;
@@ -163,17 +217,15 @@ update_switchspell_selection(PidginConversation *gtkconv)
 
 	curlang = g_strdup(g_object_get_data(G_OBJECT(gtkconv->entry), PROP_LANG));
 
-	g_list_foreach(gtk_container_get_children(GTK_CONTAINER(menu)),
-				(GFunc)gtk_check_menu_item_set_active, GINT_TO_POINTER(FALSE));
-
-	for (item = gtk_container_get_children(GTK_CONTAINER(menu));
-				item; item = item->next) {
+	items = gtk_container_get_children(GTK_CONTAINER(menu));
+	for (item = items; item; item = item->next) {
 		const char *lang = g_object_get_data(G_OBJECT(item->data), "lang");
 		if (lang && curlang && strcmp(lang, curlang) == 0) {
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item->data), TRUE);
 			break;
-		}		
+		}
 	}
+	g_list_free(items);
 	g_free(curlang);
 }
 
@@ -188,7 +240,7 @@ conversation_switched_cb(PurpleConversation *conv)
 	regenerate_switchspell_menu(gtkconv);
 	update_switchspell_selection(gtkconv);
 }
-		
+
 static gboolean
 make_sure_gtkconv(PurpleConversation *conv)
 {
